/*
   Fichier types.h
   Auteur Bernard Chardonneau

   Structure des donnes du fichier cyloop, constantes,
   accs aux bits de certaines variables.
*/


// types de donnes utilises dans le fichier cyloop

typedef unsigned char   octet;   // entier de 0  255
typedef unsigned short  uint16;  // entier de 0  65535
typedef unsigned long   uint32;  // entier de 0  4294067295
typedef short           int16;   // entier de -32768  32767
typedef long            int32;   // entier de -2147483648  2147483647


// une contante importante : nombre de secondes par jour
#define secjour      86400

// constantes boolennes
#define  OUI           1
#define  NON           0


// bits du champ caract_gen de la structure desc_cyloop

#define cycle_decl       1  // un dbut de cycle doit tre dclench par un appel de cyldecl
#define temps_utc        2  // dbut de cycles calculs en temps universel (UTC ou GMT)
#define cycle_ireg       4  // utilisation cycle de dure irrgulire bas sur le calendrier
#define cycle_annuel     8  // utilisation d'un cycle annuel


// bits du champ etat_cyloop de la structure desc_cyloop

#define heure_ete        1  // heure d't lors de la dernire mise  jour d'une variable

#define mask_heure_hiver 0xFE // complment du bit heure_ete

// #define mask_cyl_nonfait 0xF9 // complment des bits cylincr_fait et cyladdval_fait


// bits du champ typedon de la structure desc_var

#define cmoy_sep         1  // champ coef_moy distinct pour chaque instant du cycle
#define cpt_cycle_vide   2  // compter les cycles vides dans les calculs de ponderation
#define cumul_donnees    4  // autoriser plusieurs mises  jours  chaque instant du cycle
#define compteur         8  // la variable est un compteur de passages
#define donnees_32bits 0x10 // donnes sur 4 octets (significatif si compteur)
#define val_float      0x20 // donnes en virgule flottante (significatif si pas compteur)
#define mem_min_max    0x40 // mmorisation des valeurs min et max des donnes
#define cpt_toute_don  0x80 // si cumul_donnees les compteur de passage sont mis  jour chaque fois


// structure dcrivant l'entte du fichier cyloop (16 octets)

typedef struct
{
    octet  signature [3]; // Pour vrifier de c'est bien un fichier cyloop
    octet  nb_var;        // Nombre de variables mmorises dans le fichier cyloop
    octet  caract_gen;    // Caractristiques gnrales du fichier cyloop
    octet  etat_cyloop;   // Bits d'tat du fichier cyloop
    uint16 nb_don_cycle;  // Nombre de donnes par cycle de mesure
    uint32 deb_cycle;     // Instant de dmarrage du cycle courant
    uint32 duree_cycle;   // Dure en secondes du cycle courant
} desc_cyloop;


// structure dcrivant une/la variable du fichier cyloop (6 ou 8 octets)

typedef struct
{
    uint16 der_enreg;     // Numro de la dernire donne mise  jour dans le cycle actuel
    octet  typedon;       // Caractristiques de la variable
    octet  ponderation;   // Coefficient de pondration pour les valeurs des cycles prcdents
    uint16 extention;     // Champ disponible pour utilisation ultrieure
    uint16 coef_moy;      // Champ optionnel pour calcul valeur moyenne des donnes
} desc_var;
